require 'spec_helper'

module Jisota
  describe FileScript do
    describe "#initialization" do
      it "initializes with from and to" do
        script = FileScript.new(from: "foo", to: "bar")
        expect(script.from).to eq("foo")
        expect(script.to).to eq("bar")
      end

      it "accepts create and update args" do
        script = FileScript.new(from: "foo", to: "bar", create: true, update: :foo)
        expect(script.create).to be true
        expect(script.update).to eq(:foo)
      end
    end

    describe "#execute" do
      let(:script) { FileScript.new(from: "foo", to: "bar") }
      let(:ssh_session) { instance_double(SSHSession, upload: nil, command: 0) }
      let(:context) { ScriptContext.new(ssh_session: ssh_session) }

      it "uploads and moves the file" do
        allow(ssh_session).to receive(:command).with(a_string_starting_with("cmp")) { 1 }
        result = script.execute(context)

        expect(ssh_session).to have_received(:command).with("mkdir -p tmp/jisota", anything)
        expect(ssh_session).to have_received(:upload) do |options|
          expect(options[:from]).to eq("foo")
          expect(options[:to]).to start_with("tmp/jisota/")
        end
        expect(ssh_session).to have_received(:command).with(a_string_matching(/sudo mkdir -p .* sudo mv .* bar/), anything)
        expect(result).to be true
      end

      it "does not move file if it already exists and is identical" do
        allow(ssh_session).to receive(:command).with(a_string_starting_with("cmp")) { 0 }
        result = script.execute(context)

        expect(ssh_session).to_not have_received(:command).with(a_string_matching(/sudo mv/), anything)
      end

      it "does not move file when target does not exist and create is false" do
        script.create = false
        allow(ssh_session).to receive(:command).with(a_string_starting_with("cmp")) { 2 }
        result = script.execute(context)

        expect(ssh_session).to_not have_received(:command).with(a_string_matching(/sudo mv/), anything)
      end

      it "does not move file when target exists and update is false" do
        script.update = false
        allow(ssh_session).to receive(:command).with(a_string_starting_with("cmp")) { 1 }
        result = script.execute(context)

        expect(ssh_session).to_not have_received(:command).with(a_string_matching(/sudo mv/), anything)
      end

      it "runs the callbacks" do
        allow(ssh_session).to receive(:command).with(a_string_starting_with("cmp")) { 2 }
        script.create = ScriptBlock.new { cmd "foo" }
        result = script.execute(context)
        expect(ssh_session).to have_received(:command).with("foo", anything)
      end
    end

    describe "DSL" do
      it "can set simple values on callback" do
        script = FileScript.new(from: "foo", to: "bar") do
          create false
        end
        expect(script.create).to be false
      end
    end
  end
end
